
function x = bcolsweep(L, b, bs);
%==================================================================================
% Solve Lx = b, where L is lower triangular using a matrix-vector multiply.
% This is called a "block column sweep" because it is derived from the
% column sweep algorithm. If the blocksize bs is not given, bs = 4 is used.
%
% The right hand side b and solution vector x can be aliased in the calling code.
%
% This uses the convention that capital letter indices are index vectors instead
% of singleton indices. That helps make it clearer that L(J,J) is a diagonal block
% of L, and  the operation
%       x(I) = x(I) - L(I,J)*x(J);
% is a matrix*vector product. However, when mod(n,bs) = 1, the last diagonal block
% L(J,J) can actually be a singleton element L(n,n).
%==================================================================================

checklower = false; % Test for L being lower triangular or not
x = b;

% Check for dumb mistakes
if isempty(L)
    disp('Error in bcolsweep; L should be nonempty')
    x = [];
    return
end

[m,n] = size(L);
if (m~=n | length(b) ~= m)
    disp('Error in bcolsweep; L must be square and conformal with b')
    return
end

if checklower % Make sure that L is lower triangular
    up_norm = norm(triu(L,1), 1);
    if (up_norm > n*eps);
        disp('Error in bcolsweep; L must be lower triangular')
        disp(['Upper part has one-norm of ', num2str(up_norm)]);
        disp(['The cut-off limit is n*eps = ', num2str(n*eps)]);
        return
    end
end

% Set a default blocksize if one is not provided
if (nargin < 3)
    bs = 4;
end

% Number of blocks to be used
nblocks=ceil(n/bs);

for kj = 1:nblocks
    J = (kj-1)*bs+1:min(n,kj*bs); % Get the block indices
    x(J) = colsweep(L(J,J), x(J));  % <--- equivalent to x(J) = L(J,J)\b(J) 
    for ki = kj+1:nblocks
        I = (ki-1)*bs+1:min(n,ki*bs);
        x(I) = x(I) - L(I,J)*x(J);
    end
end


